/* eslint-disable no-nested-ternary */
import React, { useRef, useState, useEffect, useMemo } from 'react';
import PropTypes from 'prop-types';
import raf from 'raf';
import classNames from 'classnames/bind';
import { navsType } from '../../utils/propTypes';
import gaDataset from '../../utils/gaDataset';
// import getDataObj from '../../utils/getDataObj';
import useView from '../../hooks/view';
import AnueLink from '../../utils/anueLink';
import AnueSearch from '../AnueSearch/AnueSearch';
import AnueSubMenu from './AnueSubMenu';
// import SwitchVersionButton from './SwitchVersionButton';
import navigatorConfig from './config/navs.json';
import styles from './AnueNavigator.scss';
import { screenSizeXl, screenSizeLg, screenSizeMd } from '../../styles/_vars.scss';

const FIXED_HEADER_NONE = 'FIXED_HEADER_NONE';
const FIXED_HEADER_FULL = 'FIXED_HEADER_FULL';
const DISTANCE_OVER_HEADER = 0;

const cx = classNames.bind(styles);

function checkIsFixedHeader(fixedHeaderType) {
  return fixedHeaderType !== FIXED_HEADER_NONE;
}

function convertPxToNumber(size) {
  return parseInt(size.replace('px', ''), 10);
}

const AnueNavigator = ({
  channelName,
  navs,
  searchKeyword,
  enableSearchBar,
  dataPrefix,
  fixedHeaderType,
  accountComponent,
  switchVersionComponent,
  apiEnv,
  hideCountsWhenSearchExpand,
  isTryNewVersion,
}) => {
  const [fixed, setFixed] = useState(false);
  const [currentChannelIndex, setCurrentChannelIndex] = useState(-1);
  const [isExpandSearchInput, setIsExpandSearchInput] = useState(false);
  const [isExpandAnimationStart, setIsExpandAnimationStart] = useState(false);

  const headerRef = useRef();
  const fixedStateRef = useRef(fixed);
  const nextStateAnimationFrameIdRef = useRef();
  const tabletVerticalModeRef = useRef();
  const enableSearchInputRef = useRef(false);

  const [screenWidth] = useView();

  const anueUrl = new AnueLink(apiEnv);

  // 當畫面介在 768 ~ 1200 之間，搜尋框是縮小的
  const tabletLandscapeSize = useMemo(() => convertPxToNumber(screenSizeMd), [screenSizeMd]);
  const tabletVerticalSize = useMemo(() => convertPxToNumber(screenSizeLg), [screenSizeLg]);
  const desktopMinimalVerticalSize = useMemo(() => convertPxToNumber(screenSizeXl), [screenSizeXl]);

  tabletVerticalModeRef.current = screenWidth > tabletLandscapeSize && screenWidth <= desktopMinimalVerticalSize;

  const channels = navs != null ? navs : navigatorConfig;

  const hideCountInDesktop = hideCountsWhenSearchExpand[0] || 6;
  const hideCountInTablet = hideCountsWhenSearchExpand[1] || 4;
  const hideCounts = screenWidth <= tabletVerticalSize ? hideCountInDesktop : hideCountInTablet;

  const setNextState = state => {
    if (nextStateAnimationFrameIdRef.current) {
      raf.cancel(nextStateAnimationFrameIdRef.current);
    }

    nextStateAnimationFrameIdRef.current = raf(() => {
      nextStateAnimationFrameIdRef.current = null;
      fixedStateRef.current = state;
      setFixed(prevState => !prevState);
    });
  };

  /**
   * 監聽畫面捲動，為了處理 Heder 固定於最上端的排列
   */
  const scrollHandler = () => {
    const scrollY = 'scrollY' in window ? window.scrollY : document.documentElement.scrollTop;
    const isFixed = checkIsFixedHeader(fixedHeaderType);

    if (isFixed) {
      if (scrollY > DISTANCE_OVER_HEADER && !fixedStateRef.current) {
        setNextState(true);
      } else if (scrollY <= DISTANCE_OVER_HEADER && fixedStateRef.current) {
        setNextState(false);
      }
    }
  };

  /**
   * 點擊搜尋框的放大鏡圖示
   */
  const searchIconClickHandler = () => {
    if (!enableSearchBar) return;

    if (tabletVerticalModeRef.current) {
      enableSearchInputRef.current = !enableSearchInputRef.current;
    }
  };

  /**
   * 點擊搜尋框，啟動搜尋框展開/縮回效果
   */
  const onEnableSearchInput = enabled => {
    if (!enableSearchBar) return;

    if (tabletVerticalModeRef.current) {
      enableSearchInputRef.current = enabled;
      setIsExpandAnimationStart(true);
    }
  };

  const searchInputFocusHandler = () => {
    if (!enableSearchInputRef.current) {
      onEnableSearchInput(true);
    }
  };

  const restoreSearchInputHandler = () => {
    if (enableSearchInputRef.current) {
      onEnableSearchInput(false);
    }
  };

  /**
   * 搜尋框展開動畫效果結束
   */
  const animationEndHandler = () => {
    setIsExpandSearchInput(prevState => !prevState);
    setIsExpandAnimationStart(false);
  };

  /**
   * Accessing React State in Event Listeners with useState and useRef hooks
   * https://medium.com/geographit/accessing-react-state-in-event-listeners-with-usestate-and-useref-hooks-8cceee73c559
   */
  useEffect(() => {
    const isFixedHeader = checkIsFixedHeader(fixedHeaderType);

    if (isFixedHeader) {
      if (typeof window !== 'undefined') {
        window.addEventListener('scroll', scrollHandler);
        // The clicked event of AnueSearch is set to stopPropagation.
        // So click the anywhere outside of AnueSearch will hide the AnueRealtimeResult.
        window.addEventListener('click', restoreSearchInputHandler);
      }
    }

    return () => {
      if (isFixedHeader) {
        if (typeof window !== 'undefined') {
          window.removeEventListener('scroll', scrollHandler);
          window.removeEventListener('click', restoreSearchInputHandler);
        }
      }
    };
  }, []);

  const channelMouseEnterHandler = index => setCurrentChannelIndex(index);

  const renderNavigatorChannels = () =>
    channels.map((channel, index) => {
      if (!channel.enable) return null;

      const haveSubMenus = channel.subItems.length > 0;
      const isActive = index === currentChannelIndex;
      const needHide = enableSearchInputRef.current && index < hideCounts;
      const isCurrentNav = channelName === channel.title;
      const link =
        channel.url !== ''
          ? isTryNewVersion && channel.title === '首頁'
            ? anueUrl.generateLink(`${channel.url}/home`)
            : anueUrl.generateLink(channel.url)
          : '';

      const channelRef = useRef();

      return (
        <React.Fragment key={channel.name}>
          <li
            ref={channelRef}
            className={cx({ hide: needHide, active: isActive, current: isCurrentNav })}
            onMouseEnter={() => channelMouseEnterHandler(index)}
            onMouseLeave={() => channelMouseEnterHandler(-1)}
          >
            <div>
              {needHide ? (
                <span>{channel.title}</span>
              ) : link !== '' ? (
                <a
                  href={link}
                  target={channel.blank ? '_blank' : '_self'}
                  rel="noopener noreferrer"
                  {...gaDataset({ dataPrefix, category: 'Nav', action: 'click', label: channel.title })}
                >
                  <span className={cx({ new: channel.isNew })}>{channel.title}</span>
                </a>
              ) : (
                <span className={cx('no-link')}>{channel.title}</span>
              )}
            </div>
            {haveSubMenus && (
              <AnueSubMenu
                key={`${channel.name}_sub_menus`}
                headerRef={headerRef}
                parentRef={channelRef}
                data={channel.subItems || []}
                expandSearchBar={isExpandSearchInput}
                apiEnv={apiEnv}
              />
            )}
          </li>
        </React.Fragment>
      );
    });

  let searchInputWrapStyle = null;
  let searchInputStyle = null;
  let channelUnfoldStyle = null;

  // 1024 尺寸時
  if (tabletVerticalModeRef.current) {
    searchInputWrapStyle = enableSearchInputRef.current
      ? {
          position: 'absolute',
          right: 0,
          width: '314px',
          maxWidth: '314px',
          transition: 'width 0.2s',
        }
      : null;

    // 利用 text-indent 來達成隱藏 placeholder 的效果
    searchInputStyle = enableSearchInputRef.current ? { textIndent: '2px' } : null;
    channelUnfoldStyle = enableSearchInputRef.current
      ? cx('moveLeft', 'expansion')
      : isExpandSearchInput
        ? cx('moveRight')
        : null;
  }

  return (
    <React.Fragment>
      {fixed && <div className={cx('anue-navigator-wrapper')} />}
      <nav className={cx('anue-navigator-wrapper', { sticky: fixed })}>
        <header ref={headerRef} className={cx('main-header')}>
          <div className={cx('channel-bar')}>
            <a
              href="https://www.cnyes.com/"
              className={cx('logo')}
              {...gaDataset({ dataPrefix, category: 'Logo', action: 'click', label: 'home' })}
            />
            <div className={cx('channel-items', { mask: isExpandAnimationStart })}>
              <ul className={channelUnfoldStyle} onAnimationEnd={animationEndHandler}>
                {renderNavigatorChannels()}
              </ul>
            </div>
          </div>
          {enableSearchBar && (
            <div className={cx('search-bar')}>
              <AnueSearch
                apiEnv={apiEnv}
                enableNewStyle
                placeholder="搜尋新聞、行情代碼或名稱"
                shouldAlwaysDisplayInput
                onIconClick={searchIconClickHandler}
                onInputFocus={searchInputFocusHandler}
                customWrapperStyles={searchInputWrapStyle}
                customInputStyles={searchInputStyle}
                defaultValue={searchKeyword}
              />
            </div>
          )}
          <div className={cx('info-bar')}>
            {accountComponent && accountComponent}
            {switchVersionComponent && switchVersionComponent}
          </div>
        </header>
      </nav>
    </React.Fragment>
  );
};

AnueNavigator.propTypes = {
  channelName: PropTypes.string,
  // TODO: 之前舊版的 header 是宣告一個 Link 的屬性，用來解決編譯為靜態頁面後的連結失效問題
  // channelComponent: PropTypes.oneOfType(PropTypes.element, PropTypes.func),
  navs: navsType,
  searchKeyword: PropTypes.string,
  dataPrefix: PropTypes.arrayOf(PropTypes.string),
  enableSearchBar: PropTypes.bool,
  fixedHeaderType: PropTypes.oneOf([FIXED_HEADER_NONE, FIXED_HEADER_FULL]),
  accountComponent: PropTypes.element,
  switchVersionComponent: PropTypes.element,
  apiEnv: PropTypes.string.isRequired,
  hideCountsWhenSearchExpand: PropTypes.arrayOf(PropTypes.number),
  // 設定目前是否為新版本體驗中，對應的首頁連結會不同
  isTryNewVersion: PropTypes.bool,
};

AnueNavigator.defaultProps = {
  channelName: '首頁',
  navs: null,
  searchKeyword: '',
  // channelComponent: null,
  dataPrefix: ['data-global-ga'],
  enableSearchBar: true,
  accountComponent: null,
  switchVersionComponent: null,
  fixedHeaderType: FIXED_HEADER_FULL,
  hideCountsWhenSearchExpand: [7, 4],
  isTryNewVersion: false,
};

export default AnueNavigator;
