/* eslint-disable react/no-array-index-key */
/* eslint-disable no-unused-expressions */

/**
 * https://www.cnblogs.com/slongs/p/10189439.html
 */
import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames/bind';
import ArrowLeft from '../../assets/icon-slider-arrow-left.svg';
import ArrowRight from '../../assets/icon-slider-arrow-right.svg';
import styles from './Carousel.scss';

const cx = classNames.bind(styles);

export const DOT_POSITION = {
  FLOAT: 'float',
  FIXED: 'fixed',
};
const DELAY_TIME = 10000;
const LEFT = 'left';
const RIGHT = 'right';

/**
 * @param {number} [width = 300] - 顯示寬度
 * @param {number} [height = 193] - 顯示高度
 * @param {boolean} [hasArrow = false] - 是否需要顯示左右箭頭
 * @param {boolean} [autoPlay = true] - 是否自動播放
 * @param {number} [selected = 0] - 指定從第幾個 Slide 開始播
 * @param {number}} [speed = 10000] - 切換速度(ms)，預設是 10 秒
 * @param {string}} [dotPosition = fixed] - 分頁按鈕對齊方式，共有二種 float | fixed
 */

class CarouselOld extends React.Component {
  static propTypes = {
    width: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
    height: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
    hasArrow: PropTypes.bool,
    autoPlay: PropTypes.bool,
    children: PropTypes.ReactNode.isRequired,
    selected: PropTypes.number,
    speed: PropTypes.number,
    dotPosition: PropTypes.string,
  };

  static defaultProps = {
    width: 300,
    height: 193,
    hasArrow: true,
    autoPlay: true,
    selected: 1,
    speed: DELAY_TIME,
    dotPosition: DOT_POSITION.FLOAT,
  };

  constructor(props) {
    super(props);

    this.state = {
      currentCarousel: 0,
      animationStep: props.selected || 0,
      isShowArrow: false,
    };

    // 取得目前輪播物件個數
    this.childrenCount = React.Children.count(props.children) || 0;
  }

  /**
   * @description 開始執行
   */
  componentDidMount() {
    const { autoPlay } = this.props;

    autoPlay && this.startCarousel();
  }

  componentDidUpdate(prevProps) {
    if (prevProps.autoPlay !== this.props.autoPlay) {
      if (!this.props.autoPlay) {
        this.stopCarousel();
      } else {
        this.startCarousel();
      }
    }
  }

  componentWillUnmount() {
    this.stopCarousel();
  }

  /**
   * @description 點擊指定輪播項目切換
   */
  onGoToCarousel = index => {
    this.setState({
      currentCarousel: index,
    });
  };

  /**
   * @description 判斷導航小點點的 active 狀態
   */
  getIndicatorsActive = index => {
    const { currentCarousel } = this.state;

    return (
      currentCarousel === index ||
      currentCarousel === index + this.childrenCount ||
      (currentCarousel < 0 && index === this.childrenCount - 1)
    );
  };

  /**
   * @description 停止輪播
   */
  stopCarousel = () => {
    this.timerID && clearInterval(this.timerID);
  };

  /**
   * @description 開始輪播
   */
  startCarousel = () => {
    const { speed } = this.props;

    this.stopCarousel();
    this.timerID = setInterval(() => {
      this.handlerCarousel();
    }, speed);
  };

  /**
   * @description 顯示/隱藏箭頭
   */
  handleMouseEnter = () => {
    this.props.hasArrow && this.setState({ isShowArrow: true });
  };

  handleMouseLeave = () => {
    this.props.hasArrow && this.setState({ isShowArrow: false });
  };

  /**
   * @description 監聽動畫結束
   */
  handleTransitionEnd = () => {
    const { currentCarousel } = this.state;

    // 當輪播最後一個時，取消過場動畫，並將座標位置重設為 0
    if (currentCarousel % (this.childrenCount + 1) === this.childrenCount) {
      this.setState({
        animationStep: 0,
        currentCarousel: 0,
      });
      // 當輪播第一個時，取消過場動畫，並將座標位置設為最後一個
    } else if (currentCarousel < 0) {
      this.setState({
        animationStep: 0,
        currentCarousel: this.childrenCount - 1,
      });
    }
  };

  /**
   * @description 設定點擊箭頭後，切換顯示對應方向的 Slide 內容
   *
   * @param {string} direction - 移動方向 (left | right)
   */
  handleArrowClick = direction => {
    if (this.props.autoPlay) {
      this.startCarousel();
    }

    this.handlerCarousel(direction);
  };

  /**
   * @description 更改目前循環播放的下一個座標
   * @param {string} direction 輪播方向 (left | right)
   */
  handlerCarousel = (direction = RIGHT) => {
    const { selected } = this.props;
    const { currentCarousel } = this.state;
    const directionValue = direction === LEFT ? -1 : 1;

    // 座標不為 -1 或是目前不是最後一個的情況下，進行遞增或遞減
    if (currentCarousel % (this.childrenCount + 1) !== this.childrenCount && currentCarousel >= 0) {
      // 當座標在邊界時，會取消動畫過場的秒數，當目前座標不是邊界時，要恢復動畫過場秒數，不然無法正確播放動畫
      this.setState(prevState => {
        const current = prevState.currentCarousel + directionValue;

        return {
          animationStep: selected,
          currentCarousel: current % (this.childrenCount + 1),
        };
      });
    }
  };

  /**
   * @description 繪制箭頭
   *
   * @param {string} direction - 移動方向 (left | right)
   */
  renderCarouselArrow = (direction = LEFT) => {
    const { isShowArrow } = this.state;
    const arrowIcon = direction === LEFT ? ArrowLeft : ArrowRight;

    return (
      <div
        className={cx('carousel__arrows__arrow', { 'carousel__arrows__arrow--active': isShowArrow })}
        onClick={() => this.handleArrowClick(direction)}
      >
        <img src={arrowIcon} alt={`${direction} arrow`} />
      </div>
    );
  };

  /**
   * @description 繪製輪播項目
   *
   * @param {ReactNode} child - 輪播項目
   * @param {number} index - 索引值
   */
  renderCarouselItem = (child, index = 0) => {
    if (!child) return null;

    const totalCount = this.childrenCount + 2;
    const carouselItemStyle = {
      width: `${100 / totalCount}%`,
    };

    return (
      <div className={cx('carousel__body__item')} style={carouselItemStyle} key={`carousel_item_${index}`}>
        {child}
      </div>
    );
  };

  renderIndicators = () => {
    const { dotPosition, children } = this.props;

    if (!children) return null;

    return (
      <div className={cx('footer', { float: dotPosition === DOT_POSITION.FLOAT })}>
        <ul className={cx('footer__indicators')}>
          {children.map((_, index) => {
            const isActive = this.getIndicatorsActive(index);

            return (
              <li
                key={`carousel_indicator_${index}`}
                onClick={() => this.onGoToCarousel(index)}
                className={cx('footer__indicators__indicator', { 'footer__indicators__indicator--active': isActive })}
              />
            );
          })}
        </ul>
      </div>
    );
  };

  render() {
    const { width, height, hasArrow, children } = this.props;
    const { animationStep, currentCarousel } = this.state;

    const totalCount = this.childrenCount + 2;
    const containerWidth = width.toString().includes('%') ? width : `${width}px`;
    const containerHeight = height.toString().includes('%') ? height : `${height}px`;
    const carouselStyle = { width: containerWidth, height: containerHeight };
    const transitionStyle = {
      transition: `transform ${animationStep}s`,
      width: `${totalCount * 100}%`,
      height: containerHeight,
      transform: `translateX(${-100 / totalCount * (currentCarousel + 1)}%)`,
    };
    const firstChild = children ? children[0] : null;
    const lastChild = children ? children[this.childrenCount - 1] : null;

    return (
      <div className={cx('carousel-container')} style={carouselStyle}>
        <div className={cx('carousel')} onMouseEnter={this.handleMouseEnter} onMouseLeave={this.handleMouseLeave}>
          <div className={cx('carousel__body')} onTransitionEnd={this.handleTransitionEnd} style={transitionStyle}>
            {this.renderCarouselItem(lastChild, 0)}
            {children && children.map((item, index) => this.renderCarouselItem(item, index + 1))}
            {this.renderCarouselItem(firstChild, totalCount)}
          </div>
          <div className={cx('carousel__arrows')}>
            {hasArrow && this.renderCarouselArrow(LEFT)}
            {hasArrow && this.renderCarouselArrow(RIGHT)}
          </div>
        </div>
        {this.renderIndicators()}
      </div>
    );
  }
}

export default CarouselOld;
