/** unit test symbol list
 *
| symbol            | objectType | market |  mType  |  type  | link |
| :---------------- | :--------: | :----: | :-----: | :----: |:---:|
| TWS:TSE01:INDEX   |   QUOTE    |  TWS   |  INDEX  | INDEX  |  invest.com/index/TWS/TSE01      |
| TWS:00632R:STOCK  |   QUOTE    |  TWS   |  STOCK  |  ETF   |  invest.com/twstock/TWS/00632R   |
| TWS:2330:STOCK    |   QUOTE    |  TWS   |  STOCK  | COMMON |  invest.com/twstock/TWS/2330     |
| TWG:1269:STOCK    |   QUOTE    |  TWS   |  STOCK  | COMMON |  invest.com/twstock/TWG/1269     |
| GI:TPFI:INDEX     |   QUOTE    |   GI   |  INDEX  | INDEX  |  invest.com/index/GI/TPFI        |
| USS:SPYB:STOCK    |   QUOTE    |  USS   |  STOCK  |  ETF   |  invest.com/ETF/US/detail/SPYB   |
| USS:AAPL:STOCK    |   QUOTE    |  USS   |  STOCK  | COMMON |  invest.com//usstock/detail/AAPL |
| FX:USDTWD:FOREX   |   QUOTE    |   FX   |  FOREX  | FOREX  |  invest.com/forex/detail/USDTWD  |
| GF:YM:FUTURES     |   QUOTE    |   GF   | FUTURES | FOREX  |  invest.com/futures/GF/YM        |
| HKS:1310:STOCK    |   QUOTE    |   HKS  | STOCK   | COMMON |  global-stock/market/HKS:1310:STOCK        |
| CC:BTC:FOREX      |   QUOTE    |   CC   |  FOREX  | (TBD)  |  --                              |
| B20,073           |   FUND     |  FUND* |    -    |  null  |  --                              |
 *
 */

import AnueLink from '../anueLink';

describe('AnueLink', () => {
  let makeAnueLink;

  beforeEach(() => {
    makeAnueLink = () => AnueLink('prod');
  });

  it('snapshot - make sure all links', () => {
    const anueLink = makeAnueLink();

    expect(anueLink).toMatchSnapshot();
  });

  describe('generateStockLinkBySymbol', () => {
    it('should return empty if symbol is not completed', () => {
      const anueLink = makeAnueLink();
      const symbol = 'TWS:TSE01:';

      expect(anueLink.generateStockLinkBySymbol(symbol)).toBe('');
    });

    it('symbol: TWS:TSE01:INDEX ', () => {
      const anueLink = makeAnueLink();
      const symbol = 'TWS:TSE01:INDEX';

      expect(anueLink.generateStockLinkBySymbol(symbol)).toBe(`${anueLink.invest}/index/TWS/TSE01`);
    });

    it('symbol: TWS:00632R:STOCK', () => {
      const anueLink = makeAnueLink();
      const symbol = 'TWS:00632R:STOCK';

      expect(anueLink.generateStockLinkBySymbol(symbol)).toBe(`${anueLink.invest}/twstock/TWS/00632R`);
    });

    it('symbol: TWG:1269:STOCK', () => {
      const anueLink = makeAnueLink();
      const symbol = 'TWG:1269:STOCK';

      expect(anueLink.generateStockLinkBySymbol(symbol)).toBe(`${anueLink.invest}/twstock/TWG/1269`);
    });

    it('symbol: GI:TPFI:INDEX', () => {
      const anueLink = makeAnueLink();
      const symbol = 'GI:TPFI:INDEX';

      expect(anueLink.generateStockLinkBySymbol(symbol)).toBe(`${anueLink.invest}/index/GI/TPFI`);
    });

    it('symbol: USS:SPYB:STOCK, type: ETF', () => {
      const anueLink = makeAnueLink();
      const symbol = 'USS:SPYB:STOCK';
      const type = 'ETF';

      expect(anueLink.generateStockLinkBySymbol(symbol, type)).toBe(`${anueLink.invest}/ETF/US/detail/SPYB`);
    });

    it('symbol: USS:AAPL:STOCK', () => {
      const anueLink = makeAnueLink();
      const symbol = 'USS:AAPL:STOCK';

      expect(anueLink.generateStockLinkBySymbol(symbol)).toBe(`${anueLink.invest}/usstock/detail/AAPL`);
    });

    it('symbol: GF:YM:FUTURES', () => {
      const anueLink = makeAnueLink();
      const symbol = 'GF:YM:FUTURES';

      expect(anueLink.generateStockLinkBySymbol(symbol)).toBe(`${anueLink.invest}/futures/GF/YM`);
    });

    it('symbol: HKS:1310:STOCK', () => {
      const anueLink = makeAnueLink();
      const symbol = 'HKS:1310:STOCK';

      expect(anueLink.generateStockLinkBySymbol(symbol)).toBe(`${anueLink.globalStockTalk}/market/HKS:1310:STOCK`);
    });

    it('symbol: FX:USDTWD:FOREX', () => {
      const anueLink = makeAnueLink();
      const symbol = 'FX:USDTWD:FOREX';

      expect(anueLink.generateStockLinkBySymbol(symbol)).toBe(`${anueLink.invest}/forex/detail/USDTWD/overview`);
    });

    it('symbol: CC:BTC:FOREX', () => {
      const anueLink = makeAnueLink();
      const symbol = 'CC:BTC:FOREX';

      expect(anueLink.generateStockLinkBySymbol(symbol)).toBe(`${anueLink.cryptoChannelLink}/BTC`);
    });
  });

  describe('generateFundLink', () => {
    it('return fund link', () => {
      const anueLink = makeAnueLink();
      const id = 'B22,068';
      const displayName = '<mark>駿利亨德森資產管理基金-駿利亨德森美國策略價值基金A2美元</mark>';

      expect(anueLink.generateFundLink(displayName, id)).toBe(
        `${anueLink.driverFundDetailLink}/駿利亨德森資產管理基金-駿利亨德森美國策略價值基金A2美元/B22,068`
      );
    });
  });

  describe('generateETFLink', () => {
    it('return empty string if symbol is not completed', () => {
      const anueLink = makeAnueLink();
      const symbol = 'USS';

      expect(anueLink.generateETFLink(symbol)).toBe('');
    });

    it('return empty string if symbol is illegal', () => {
      const anueLink = makeAnueLink();
      const symbol = 'TWG:1269:STOCK';

      expect(anueLink.generateETFLink(symbol)).toBe('');
    });

    it('return empty string if symbol is empty', () => {
      const anueLink = makeAnueLink();
      const symbol = '';

      expect(anueLink.generateETFLink(symbol)).toBe('');
    });

    it('symbol: TWS:00632R:STOCK', () => {
      const anueLink = makeAnueLink();
      const symbol = 'TWS:00632R:STOCK';

      expect(anueLink.generateETFLink(symbol)).toBe(`${anueLink.stockTalk}/market/TWS:00632R:STOCK`);
    });

    it('symbol: USS:SPYB:STOCK', () => {
      const anueLink = makeAnueLink();
      const symbol = 'USS:SPYB:STOCK';

      expect(anueLink.generateETFLink(symbol)).toBe(`${anueLink.invest}/etf/us/detail/SPYB`);
    });
  });

  describe('generateSearchResultLink', () => {
    it('return search link', () => {
      const anueLink = makeAnueLink();
      const param = { keyword: 'TEST', backLink: 'invest.com' };

      expect(anueLink.generateSearchResultLink(param)).toBe(
        `${anueLink.wwwChannelLink}/search/all?keyword=TEST&aink=invest.com`
      );
    });
  });

  describe('generateSearchResultLink', () => {
    it('return search link', () => {
      const anueLink = makeAnueLink();
      const param = { keyword: 'TEST', backLink: 'invest.com' };

      expect(anueLink.generateSearchResultLink(param)).toBe(
        `${anueLink.wwwChannelLink}/search/all?keyword=TEST&aink=invest.com`
      );
    });
  });

  describe('generateForexLink', () => {
    it('return empty string if symbol is not completed', () => {
      const anueLink = makeAnueLink();
      const symbol = 'FX';

      expect(anueLink.generateForexLink(symbol)).toBe('');
    });

    it('return empty string if symbol is not FX', () => {
      const anueLink = makeAnueLink();
      const symbol = 'TWG:1269:STOCK';

      expect(anueLink.generateForexLink(symbol)).toBe('');
    });

    it('return empty string if symbol is empty', () => {
      const anueLink = makeAnueLink();
      const symbol = '';

      expect(anueLink.generateForexLink(symbol)).toBe('');
    });

    it('symbol: FX:USDTWD:FOREX', () => {
      const anueLink = makeAnueLink();
      const symbol = 'FX:USDTWD:FOREX';

      expect(anueLink.generateForexLink(symbol)).toBe(`${anueLink.invest}/forex/detail/USDTWD/overview`);
    });
  });

  describe('generateFundSaleLink', () => {
    it('return empty string if forSale is not equal to 1', () => {
      const anueLink = makeAnueLink();
      const forSale = 2;
      const displayNameLocal = '景順全歐洲企業基金A股 歐元';
      const cnyesId = 'B16,009';

      expect(anueLink.generateFundSaleLink({ forSale, displayNameLocal, cnyesId })).toBe('');
    });

    it('return fund sale link', () => {
      const anueLink = makeAnueLink();
      const forSale = 1;
      const displayNameLocal = '景順全歐洲企業基金A股 歐元';
      const cnyesId = 'B16,009';

      expect(anueLink.generateFundSaleLink({ forSale, displayNameLocal, cnyesId })).toBe(
        `${anueLink.invest}/funds/order/景順全歐洲企業基金A股 歐元/B16,009`
      );
    });

    it('return fund sale link with utm', () => {
      const anueLink = makeAnueLink();
      const forSale = 1;
      const displayNameLocal = '景順全歐洲企業基金A股 歐元';
      const cnyesId = 'B16,009';
      const source = 'cnyes';
      const medium = 'myprofile_fundlist';
      const campaign = 'buy-myprofilepage';

      expect(anueLink.generateFundSaleLink({ forSale, displayNameLocal, cnyesId, source, medium, campaign })).toBe(
        `${
          anueLink.invest
        }/funds/order/景順全歐洲企業基金A股 歐元/B16,009?utm_source=cnyes&utm_medium=myprofile_fundlist&utm_campaign=buy-myprofilepage`
      );
    });
  });
});
